 sdkms-cli

`sdkms-cli` is a simple, cross-platform SDKMS client command line utility
written in Python.

## API endpoint
By default, sdkms-cli makes REST API calls to the SDKMS server at
https://apps.sdkms.fortanix.com. To make calls to a different SDKMS server,
either set the environment variable SDKMS_API_ENDPOINT, e.g.,
`SDKMS_API_ENDPOINT=<sdkms-server-url>`, or provide the
server url on command line using the `--api-endpoint` option.

## Authentication

The Fortanix SDKMS Server supports two types of authentication: user
authentication and application authentication. Users are authenticated
with passwords. Applications are authenticated with an API Key or with
a certificate.

Each action that may be performed in SDKMS requires some form of authorization.
Some actions, such as encrypting or decrypting data, may only be performed
with Application credentials. Some actions, like creating new Groups,
creating Applications, and changing group membership, may only be done
with User credentials. Some actions, such as creating new Security Objects
(keys) may be performed with either User credentials or Application credentials.

sdkms-cli supports both user authentication and application authentication.
It may have an active session for a user, an application, or both simultaneously.

https://support.fortanix.com/developers-guide-concepts.html

`sdkms-cli user-login` authenticates a user account using its SDKMS username
and password.

`sdms-client app-login` authenticates an application with an API key.

In both cases, persistent session information is stored in in a hidden file
`.token` in the current directory.

`sdkms-cli logout` logs out of the SDKMS server and clears `.token`.
You may also use `sdkms-cli user-logout` or `sdkms-cli app-logout`
to log out of just the current user session or just the current application
session.

For example,

```
$ sdkms-cli user-login
Please enter your SDKMS username: user@example.com
SDKMS Password:
Successfully logged in
$ sdkms-cli list-apps
bf105894-f004-4c75-b4ba-f51168674355 test app This is a test application Groups(5893a573-ed9c-4e38-a3bc-ad72a1820bc1)
$ sdkms-cli app-login
Please enter your API Key: <paste an SDKMS API Key>
Successfully logged in
$ sdkms-cli logout
User logged out
App logged out
```

## Importing keys and certificates

`sdkms-cli import-key` and `sdkms-cli import-cert` expect
keys and certificates to be in PEM format.

## Example commands

```
$ sdkms-cli user-login
Please enter your SDKMS username:
SDKMS Password:
Successfully logged in
$ sdkms-cli create-group --name "My group" --description "Test group"
47e173b6-c993-476e-af3f-013509428c78
$ sdkms-cli list-groups
47e173b6-c993-476e-af3f-013509428c78 "My group" "Test group" 7bbb7a0b-de6b-4311-a7fa-053de7eabced
$ sdkms-cli create-app --name "My app" --description "Test app" --groups 47e173b6-c993-476e-af3f-013509428c78
2b9a6296-3ed7-47a6-8ab2-5ddf92935f6e
$ sdkms-cli list-apps
2b9a6296-3ed7-47a6-8ab2-5ddf92935f6e "My app" "Test app" Groups(47e173b6-c993-476e-af3f-013509428c78)
$ sdkms-cli get-app-api-key --name "My app"
O522CZVDF5BwQHaG+hP3K/HH+uYLgEKoUDCCKOtVa7YFkluhkCZFwPK1wVnpz8lE5L+msHAHT6hLjAGKTB++kyOoT0kbG5TyiqUl1kZTnjhU4JkOAyLfdqcq2D8MsCWbYgjVOsgmMcPo4o28eNetQuzW+DnDyueHrH29e0
$ sdkms-cli app-login
Please enter your API Key: O522CZVDF5BwQHaG+hP3K/HH+uYLgEKoUDCCKOtVa7YFkluhkCZFwPK1wVnpz8lE5L+msHAHT6hLjAGKTB++kyOoT0kbG5TyiqUl1kZTnjhU4JkOAyLfdqcq2D8MsCWbYgjVOsgmMcPo4o28eNetQuzW+DnDyueHrH29e0
Successfully logged in
$ sdkms-cli create-key --name "New key" --obj-type RSA --key-size 2048 --custom-metadata '{ "zone" : "West" }'
Created Key "New key"
$ sdkms-cli import-key --name "Existing key" --in existing_key.key --obj-type RSA
Imported key "Existing key"
$ sdkms-cli list-keys
3d9065b2-e847-4b9a-ba31-8b1f5545004a "New key" Created by sdkms-cli 2048 RSA
7a0dc06a-0646-415a-8cf4-87576e14f940 "Existing key" Created by sdkms-cli 2048 RSA
$ sdkms-cli show-object --kid 3d9065b2-e847-4b9a-ba31-8b1f5545004a
id: 3d9065b2-e847-4b9a-ba31-8b1f5545004a
name: New key
description: Created by sdkms-cli
type: RSA
key size: 2048
origin: FortanixHSM
operations: SIGN, VERIFY, ENCRYPT, DECRYPT, WRAPKEY, UNWRAPKEY, DERIVEKEY
custom metadata:
{
	"zone": "West"
} // end custom metadata
$ sdkms-cli delete-key --kid 7a0dc06a-0646-415a-8cf4-87576e14f940
Deleted Key 7a0dc06a-0646-415a-8cf4-87576e14f940
```
# SDKMS CLI Man Page

SDKMS-CLI(1)                                                   User Commands                                                  SDKMS-CLI(1)

NAME
       sdkms-cli - manual page for sdkms-cli

DESCRIPTION
       usage: sdkms_cli [-h] [--version] [--api-endpoint API_ENDPOINT] 
       [--debug-http] [--debug-errors] [--prefer-user-auth] [--prefer-app-auth] 
       [--no-verify-ssl] [--verify-ssl] [--proxy PROXY_URL] CMD

CMD: 

	app-login
              Log in to SDKMS with an application API key for performing crypto operations and key management

       user-login
              Log in to SDKMS with a username and password for performing crypto operations and key management

       app-logout
              Log out of application credentials

       user-logout
              Log out of user credentials

       select-account
              switch to another account

       logout Log out of both application and user credentials

       rotate-key
              rotate a key

       create-key
              create a key

       persist-key
              persist a transient key

       import-key
              Import a key into SDKMS

       import-cert
              import a certificate

       import-secret
              Import a secret into SDKMS

       update-object
              Update metadata of a security object

       delete-key
              Delete a key

       list-keys
              List keys

       list-objects
              List objects (keys, certificates, and other opaque objects)

       show-sobject
              Show details about a Security Object

       export-object
              Show value of a Security Object. This works only if the object type is Secret.

       activate-object
              Activate a Security Object. This works only if the object is in Preactive state.

       revoke-object
              Revoke a Security Object.

       encrypt
              Encrypt with key

       encrypt-init
              Multipart encryption initialization

       encrypt-update
              Multipart encryption update

       encrypt-final
              Multipart encryption final

       decrypt-init
              Multipart decryption initialization

       decrypt-update
              Multipart decryption update

       decrypt-final
              Multipart encryption final

       decrypt
              Decrypt with key

       wrap-key
              Wrap a key with another key

       unwrap-key
              Unwrap a key that has been wrapped with another key

       derive-key
              Derive a new key from existing (base) key.

       list-accounts
              List associated accounts

       list-apps
              List Applications in SDKMS

       create-app
              Create a new Application in SDKMS

       get-app-api-key
              Get the API key for an Application. Specify the application with --name or --app-id (but not both)

       regenerate-app-api-key
              Reset the API key for an Application. This will invalidate any previous API keys. Specify the application with --name
              or --app-id (but not both)

       delete-app
              Delete an Application. The Application to be deletedmay be specified by name or by app id. If both name  and  app  id
              are provided, the Application will only be deleted if both name and app id match

       list-groups
              List groups in SDKMS

       create-group
              Create a new security group

       delete-group
              Delete a security group.

       sign   Signature generation with RSA and EC keys

       batch-sign
              Signature generation with RSA and EC keys in batch

       batch-encrypt
              Encryption in batch

       batch-decrypt
              Decryption in batch

       batch-verify
              Signature verification with RSA and EC keys in batch

       verify Signature verification with RSA and EC keys

       message-digest
              Message Digest

       hmac-digest
              Compute HMAC digest. Computed digest is hex encoded

       hmac-verify
              Compute HMAC digest

       agree-key
              Agree key

       invoke-plugin
              Invoke sdkms plugin

       update-plugin
              Invoke sdkms update plugin

       delete-plugin
              Invoke sdkms delete plugin

       create-plugin
              Invoke sdkms update plugin

       server-version
              check sdkms server version

       server-health
              check sdkms server health

       app-certificate-login
              Log in to SDKMS with a certificate for performing account tasks

       show-app
              Display app properties

       show-group
              Display group properties

       show-user
              Display user properties

       show-account
              Display account properties

       task-result
              Display resault of an approval task

   optional arguments:
       -h, --help
              show this help message and exit

       --version
              show program's version number and exit

       --api-endpoint API_ENDPOINT
              SDKMS API endpoint to connect to

       --no-verify-ssl
              Don't verify server certificate

       --verify-ssl
              Verify server certificate

       --debug-http
              Turn on verbose HTTP request debugging

       --debug-errors
              Turn on more verbose error output

       --prefer-user-auth
              Prefer using user authentication for APIs that accept both

       --prefer-app-auth
              Prefer using app authentication for APIs that accept both

       --proxy
              URL of the proxy to be used

> Individual CMD help can be shown as 
> \# sdkms-cli <CMD\> --help
