#!/usr/bin/python
from subprocess import call, check_output, CalledProcessError
import os
import random
import tempfile
import sys

uuids = []

def get_AES_key_name():
    return "AES" + str(random.randint(1,1000000))

def get_RSA_key_name():
    return "RSA" + str(random.randint(1,1000000))

def run_command(cmd):
    print ">>> "+cmd
    return call("python sdkms-client.py "+ cmd, shell=True)

def run_command_with_output(cmd):
    print ">>> "+cmd
    return check_output("python sdkms-client.py " + cmd, shell=True)

def demo_encrypt_symmetric():
    print "\n====> Creating an AES key <===="
    cmd="create-key --obj-type AES --key-size 256 --description \"test symmetric key\" --name "
    while True:
        try:
            name = get_AES_key_name()
            uuid = run_command_with_output(cmd+str(name)).rstrip()
            if uuid:
                uuids.append(uuid)
                print 'Key name "' + name + '" created. UUID = ' + uuid
                print
                print run_command_with_output("show-object --kid " + uuid)
                break
        except CalledProcessError:
            print 'Key name "' + name + '" already exists'

    f, infile = tempfile.mkstemp()
    text = "Hello Encryption"
    os.write(f, text)
    os.close(f)
   
    f, outfile = tempfile.mkstemp()
    print "\n====> Encrypting using AES key <===="
    print "text being encrypted = " + text
    cmd="encrypt --kid " + uuid + " --alg AES --mode CBC --in " + infile + " --out " + outfile 
    run_command(cmd)
    with open(outfile, "r") as outf:
        encrypted_blob = outf.read()
    with open(outfile + ".iv", "r") as ivf:
        iv = ivf.read()
    print "encrypted blob = " + encrypted_blob + ", iv = " + iv
    os.close(f)
   
    f, outoutfile = tempfile.mkstemp()
    print "\n====> Decrypting using AES key <===="
    print "text being decrypted = " + encrypted_blob
    cmd="decrypt --kid " + uuid + " --alg AES --mode CBC --in " + outfile + " --out " + outoutfile + " --iv " + iv
    run_command(cmd)
    with open(outoutfile, "r") as outoutf:
        decrypted_text = outoutf.read()
    print "decrypted text = " + decrypted_text
    os.close(f)

    os.unlink(infile) 
    os.unlink(outfile) 
    os.unlink(outfile+".iv") 
    os.unlink(outoutfile) 

def demo_encrypt_asymmetric():
    print "\n====> Creating an RSA key <===="
    cmd="create-key --obj-type RSA --key-size 2048 --description \"test asymmetric key\" --name "
    while True:
        try:
            name = get_RSA_key_name()
            uuid = run_command_with_output(cmd+name).rstrip()
            if uuid:
                uuids.append(uuid)
                print 'Key name "' + name + '" created. UUID = ' + uuid
                print
                print run_command_with_output("show-object --kid " + uuid)
                break
        except CalledProcessError:
            print 'Key name "' + name + '" already exists'

    f, infile = tempfile.mkstemp()
    text = "Hello Encryption"
    os.write(f, text)
    os.close(f)
   
    f, outfile = tempfile.mkstemp()
    print "\n====> Encrypting using RSA key <===="
    print "text being encrypted = " + text
    cmd="encrypt --kid " + uuid + " --alg RSA --in " + infile + " --out " + outfile 
    run_command(cmd)
    with open(outfile, "r") as outf:
        encrypted_blob = outf.read()
    print "encrypted blob = " + encrypted_blob 
    os.close(f)
   
    f, outoutfile = tempfile.mkstemp()
    print "\n====> Decrypting using RSA key <===="
    print "text being decrypted = " + encrypted_blob
    cmd="decrypt --kid " + uuid + " --alg RSA --in " + outfile + " --out " + outoutfile
    run_command(cmd)
    with open(outoutfile, "r") as outoutf:
        decrypted_text = outoutf.read()
    print "decrypted text = " + decrypted_text
    os.close(f)
    
    os.unlink(infile) 
    os.unlink(outfile) 
    os.unlink(outoutfile) 

def demo_list_keys():
    print "\n====> Listing keys available to the app <===="
    cmd="list-keys"
    run_command(cmd)

def demo_wrap_unwrap():
    print "\n====> Creating a wrapping key <===="
    cmd="create-key --obj-type AES --key-size 256 --description \"wrapping key\" --name "
    while True:
        try:
            w_name = get_AES_key_name()
            w_uuid = run_command_with_output(cmd+w_name).rstrip()
            if w_uuid:
                uuids.append(w_uuid)
                print 'Key name "' + w_name + '" created. UUID = ' + w_uuid
                print
                print run_command_with_output("show-object --kid " + w_uuid)
                break
        except CalledProcessError:
            print 'Key name "' + w_name + '" already exists'
    
    print "\n====> Creating the key to be wrapped <===="
    cmd="create-key --obj-type AES --key-size 256 --exportable --description \"exportable key\" --name "
    while True:
        try:
            name = get_AES_key_name()
            uuid = run_command_with_output(cmd+name).rstrip()
            if uuid:
                uuids.append(uuid)
                print 'Key name "' + name + '" created. UUID = ' + uuid
                print
                print run_command_with_output("show-object --kid " + uuid)
                break
        except CalledProcessError:
            print 'Key name "' + name + '" already exists'

    f, outfile = tempfile.mkstemp()
    print "\n====> Wrapping key <===="
    print "wrapping key " + name + " with key AES" + w_name
    cmd="wrap-key --wrapping-kid " + w_uuid + " --kid " + uuid + " --alg AES --mode CBC --out " + outfile 
    run_command(cmd)
    with open(outfile, "r") as outf:
        encrypted_blob = outf.read()
    with open(outfile + ".iv", "r") as ivf:
        iv = ivf.read()
    print "wrapped blob = " + encrypted_blob + ", iv = " + iv
    os.close(f)
   
    print "\n====> Unwrapping key <===="
    print "unwrapping key " + name + " with key " + w_name
    name = get_AES_key_name()
    cmd="unwrap-key --wrapping-kid " + w_uuid + " --alg AES --mode CBC --in " + outfile + " --name " + name + " --obj-type AES --iv " + iv
    uuid = run_command_with_output(cmd).rstrip()
    print 'Key name "' + name + '" created. UUID = ' + uuid
    uuids.append(uuid)
    print
    print run_command_with_output("show-object --kid " + uuid)

    os.unlink(outfile) 
    os.unlink(outfile+".iv") 

def cleanup():
    for uuid in uuids:
        run_command("delete-key --kid " + uuid)
    run_command("app-logout")
    sys.exit(0)

if __name__ == "__main__":
    endpoint = raw_input("Enter API Endpoint: ")
    os.putenv("FORTANIX_API_ENDPOINT", endpoint)
    print "====> App logging in <===="
    cmd="app-login"
    if run_command(cmd) != 0:
        print "Can't login. Exiting"
        sys.exit(1)

    while True:
        print 
        print "--------------------------------------------------------------------" 
        print "Please choose the demo number to run from the following list:"
        print "1. List all keys available to the app"
        print "2. Create an AES key, and use it for encryption and decryption"
        print "3. Create an RSA key, and use it for encryption and decryption"
        print "4. Create an AES key to wrap and unwrap another AES key"
        print "0. Cleanup and exit"
        print

        demo_number = input("Enter you selection: ")
        print
        if demo_number == 0:
            cleanup()
        elif demo_number == 1:
            demo_list_keys()
        elif demo_number == 2:
            demo_encrypt_symmetric()
        elif demo_number == 3:
            demo_encrypt_asymmetric()
        elif demo_number == 4:
            demo_wrap_unwrap()
        else:
            print "Try again"

